==> ehcache_2.1.0.orig.tar.gz: ehcache-2.1.0.orig/src/test/java/net/sf/ehcache/constructs/blocking/BlockingCacheTest.java <==
/**
 *  Copyright 2003-2010 Terracotta, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package net.sf.ehcache.constructs.blocking;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheException;
import net.sf.ehcache.CacheTest;
import net.sf.ehcache.Ehcache;
import net.sf.ehcache.Element;
import net.sf.ehcache.Statistics;
import net.sf.ehcache.Status;
import net.sf.ehcache.StopWatch;
import net.sf.ehcache.config.CacheConfiguration;
import net.sf.ehcache.statistics.LiveCacheStatistics;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

/**
 * Test cases for the {@link BlockingCache}.
 *
 * @author Adam Murdoch
 * @author Greg Luck
 * @version $Id: BlockingCacheTest.java 2437 2010-05-13 17:47:38Z alexsnaps $
 */
public class BlockingCacheTest extends CacheTest {

    private static final Logger LOG = LoggerFactory.getLogger(BlockingCacheTest.class.getName());
    private BlockingCache blockingCache;

    /**
     * Load up the test cache
     */
    @Before
    public void setUp() throws Exception {
        super.setUp();
        Ehcache cache = manager.getCache("sampleIdlingExpiringCache");
        blockingCache = new BlockingCache(cache);
    }

    /**
     * teardown
     */
    @After
    public void tearDown() throws Exception {
        if (manager.getStatus() == Status.STATUS_ALIVE) {
            blockingCache.removeAll();
        }
        super.tearDown();
    }

    /**
     * Tests adding and looking up an entry.
     */
    @Test
    public void testAddEntry() throws Exception {
        //some other test was leaving this non-empty
        blockingCache.removeAll();
        
        final String key = "key";
        final String value = "value";
        Element element = new Element(key, value);

        // Check the cache is empty
        assertEquals(0, blockingCache.getKeys().size());

        // Put the entry
        blockingCache.put(new Element(key, value));

        // Check there is a single entry
        assertEquals(1, blockingCache.getKeys().size());
        assertTrue(blockingCache.getKeys().contains(key));
        final Element returnedElement = blockingCache.get(key);
        assertEquals(element, returnedElement);

    }

    /**
     * Tests that getting entries matches a list of known entries
     */
    @Test
    public void testGetEntries() throws Exception {
        Ehcache cache = blockingCache.getCache();
        for (int i = 0; i < 100; i++) {
            cache.put(new Element(Integer.valueOf(i), "value" + i));
        }
        List keys = blockingCache.getKeys();
        List elements = new ArrayList();
        for (int i = 0; i < keys.size(); i++) {
            Object key = keys.get(i);
            elements.add(blockingCache.get(key));
        }
        assertEquals(100, elements.size());
        Map map = new HashMap();
        for (int i = 0; i < elements.size(); i++) {
            Element element = (Element) elements.get(i);
            map.put(element.getObjectKey(), element.getObjectValue());
        }
        for (int i = 0; i < 100; i++) {
            Serializable value = (Serializable) map.get(Integer.valueOf(i));
            assertEquals("value" + i, value);
        }
    }

    /**
     * Tests looking up a missing entry, then adding it.
     */
    @Test
    public void testAddMissingEntry() throws Exception {
        Element element = new Element("key", "value");

        // Make sure the entry does not exist
        assertNull(blockingCache.get("key"));

        // Put the entry
        blockingCache.put(element);

        // Check the entry is in the cache
        assertEquals(1, blockingCache.getKeys().size());
        assertEquals(element, blockingCache.get("key"));

    }


    /**
     * Does a second tread block until the first thread puts the entry?
     */
    @Test
    public void testSecondThreadActuallyBlocks() throws Exception {
        Element element = new Element("key", "value");
        final List threadResults = new ArrayList();

        // Make sure the entry does not exist
        assertNull(blockingCache.get("key"));

        Thread secondThread = new Thread() {
            public void run() {
                threadResults.add(blockingCache.get("key"));
            }
        };
        secondThread.start();
        assertEquals(0, threadResults.size());

        // Put the entry
        blockingCache.put(element);
        Thread.sleep(30);
        assertEquals(1, threadResults.size());
        assertEquals(element, threadResults.get(0));

        // Check the entry is in the cache
        assertEquals(1, blockingCache.getKeys().size());
        assertEquals(element, blockingCache.get("key"));
    }

    /**
     * Elements with null valuea are not stored in the blocking cache
     */
    @Test
    public void testUnknownEntry() throws Exception {
        // Make sure the entry does not exist
        assertNull(blockingCache.get("key"));
        // Put the entry
        blockingCache.put(new Element("key", null));
        assertEquals(0, blockingCache.getKeys().size());
    }

    /**
     * Overwriting an Element with an element with a null value effectively removes it from the cache
     */
    @Test
    public void testRemoveEntry() throws Exception {
        Element element = new Element("key", "value");

        // Add entry and make sure it's there
        blockingCache.put(element);
        assertEquals(element, blockingCache.get("key"));

        // Remove the entry and make sure its gone
        blockingCache.put(new Element("key", null));
        assertEquals(0, blockingCache.getKeys().size());

    }

    /**
     * Tests clearing the cache
     */
    @Test
    public void testClear() throws Exception {
        Ehcache cache = manager.getCache("sampleCacheNotEternalButNoIdleOrExpiry");
        blockingCache = new BlockingCache(cache);
        // Add some entries
        blockingCache.put(new Element("key1", "value1"));
        blockingCache.put(new Element("key2", "value2"));
        blockingCache.put(new Element("key3", "value2"));
        assertEquals(3, blockingCache.getKeys().size());

        // Clear the cache
        blockingCache.removeAll();
        assertEquals(0, blockingCache.getKeys().size());
    }


    /**
     * Thrashes a BlockingCache and looks for liveness problems
     * Note. These timings are without logging. Turn logging off to run this test.
     */
    @Test
    public void testThrashBlockingCache() throws Exception {
        Ehcache cache = manager.getCache("sampleCache1");
        blockingCache = new BlockingCache(cache);
        long duration = thrashCache(blockingCache, 50, 500L, 1000L);
        LOG.debug("Thrash Duration:" + duration);
    }

    /**
     * Thrashes a BlockingCache which has a tiny timeout. Should throw
     * a LockTimeoutException caused by queued threads not getting the lock
     * in the required time.
     */
    @Test
    public void testThrashBlockingCacheTinyTimeout() throws Exception {
        Ehcache cache = manager.getCache("sampleCache1");
        blockingCache = new BlockingCache(cache);
        blockingCache.setTimeoutMillis(1);
        long duration = 0;
        try {
            duration = thrashCache(blockingCache, 50, 400L, 100L);
            fail();
        } catch (Exception e) {
            //expected
        }
        LOG.debug("Thrash Duration:" + duration);
    }

    /**
     * Thrashes a BlockingCache which has a reasonable timeout. Should work.
     * The old implementation, which had scalability limits, needed 5, 1000L, 5000L to pass
     */
    @Test
    public void testThrashBlockingCacheReasonableTimeout() throws Exception {
        Ehcache cache = manager.getCache("sampleCache1");
        blockingCache = new BlockingCache(cache);
        blockingCache.setTimeoutMillis((int) (400 * StopWatch.getSpeedAdjustmentFactor()));
        long duration = thrashCache(blockingCache, 50, 400L, (long) (1000L * StopWatch.getSpeedAdjustmentFactor()));
        LOG.debug("Thrash Duration:" + duration);
    }

    /**
     * This method tries to get the cache to slow up.
     * It creates 300 threads, does blocking gets and monitors the liveness right the way through
     */
    private long thrashCache(final BlockingCache cache, final int numberOfThreads,
                             final long liveness, final long retrievalTime) throws Exception {
        StopWatch stopWatch = new StopWatch();

        // Create threads that do gets
        final List executables = new ArrayList();
        for (int i = 0; i < numberOfThreads; i++) {
            final Executable executable = new Executable() {
                public void execute() throws Exception {
                    for (int i = 0; i < 10; i++) {
                        final String key = "key" + i;
                        Object value = cache.get(key);
                        checkLiveness(cache, liveness);
                        if (value == null) {
                            cache.put(new Element(key, "value" + i));
                        }
                        //The key will be in. Now check we can get it quickly
                        checkRetrievalOnKnownKey(cache, retrievalTime, key);
                    }
                }
            };
            executables.add(executable);
        }

        int failures = runThreadsNoCheck(executables, true);
        if (failures > 0) {

            throw new Exception("failures");
        }
        assertTrue("Failures: " + failures, failures <= 0);
        cache.removeAll();
        return stopWatch.getElapsedTime();
    }

    /**
     * Checks that the liveness method returns in less than a given amount of time.
     * liveness() is a method that simply returns a String. It should be very fast. It can be
     * delayed because it is a synchronized method, and must acquire an object lock before continuing
     * The old blocking cache was taking up to several minutes in production
     *
     * @param cache a BlockingCache
     */
    private void checkLiveness(BlockingCache cache, long liveness) {
        StopWatch stopWatch = new StopWatch();
        cache.liveness();
        long measuredLiveness = stopWatch.getElapsedTime();
        assertTrue("liveness is " + measuredLiveness + " but should be less than " + liveness + "ms",
                measuredLiveness < liveness);
    }

    /**
     * Checks that the liveness method returns in less than a given amount of time.
     * liveness() is a method that simply returns a String. It should be very fast. It can be
     * delayed because it is a synchronized method, and must acquire
     * an object lock before continuing. The old blocking cache was taking up to several minutes in production
     *
     * @param cache a BlockingCache
     */
    private void checkRetrievalOnKnownKey(BlockingCache cache, long requiredRetrievalTime, Serializable key)
            throws LockTimeoutException {
        StopWatch stopWatch = new StopWatch();
        cache.get(key);
        long measuredRetrievalTime = stopWatch.getElapsedTime();
        assertTrue("Retrieval time on known key is " + measuredRetrievalTime
                + " but should be less than " + requiredRetrievalTime + "ms",
                measuredRetrievalTime < requiredRetrievalTime);
    }

    /**
     * Creates a blocking test cache
     */
    protected Ehcache createTestCache() {
        Ehcache cache = super.createTestCache();
        return new BlockingCache(cache);
    }

    /**
     * Gets the sample cache 1
     */
    protected Ehcache getSampleCache1() {
        Cache cache = manager.getCache("sampleCache1");
        manager.replaceCacheWithDecoratedCache(cache, new BlockingCache(cache));
        return manager.getEhcache("sampleCache1");
    }

    /**
     * Use to manually test super class tests
     */
    @Test
    public void testInstrumented() throws Exception {
        super.testSizes();
    }

    @Test
    public void testGetWithLoader() {
        super.testGetWithLoader();
    }


    @Test
    public void testFlushWhenOverflowToDisk() throws Exception {
        super.testFlushWhenOverflowToDisk();
    }

    @Test
    public void testConcurrentPutsAreConsistentRepeatedly() throws InterruptedException {
        //do nothing
    }

    @Test
    public void testConcurrentPutsAreConsistent() throws InterruptedException {
        //do nothing
    }

    @Test
    public void testInlineEviction() throws InterruptedException {

        final Serializable KEY = "DUH";
        Cache cache = new Cache(new CacheConfiguration("fastExpiry", 1000).timeToIdleSeconds(2).timeToLiveSeconds(2));
        manager.addCache(cache);
        manager.replaceCacheWithDecoratedCache(cache, new BlockingCache(cache));

        Ehcache blockingCache = manager.getEhcache("fastExpiry");
        blockingCache.put(new Element(KEY, "VALUE"));
        assertNotNull(blockingCache.get(KEY));
        // This tests inline eviction (EHC-420)
        Thread.sleep(3000);
        assertNull(blockingCache.get(KEY));
    }

    @Test
    public void testTimeout() throws BrokenBarrierException, InterruptedException {
        final CyclicBarrier barrier = new CyclicBarrier(2);
        final String KEY = "BLOCKING_KEY";
        blockingCache.setTimeoutMillis(1000);
        Thread thread = new Thread(new Runnable() {
            public void run() {
                assertNull(blockingCache.get(KEY));
                try {
                    barrier.await();
                    Thread.sleep(5000);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
                blockingCache.put(new Element(KEY, "VALUE"));
            }
        });
        thread.start();
        barrier.await();
        try {
            blockingCache.get(KEY);
            fail("BlockingCache.get should have not returned!");
        } catch (CacheException e) {
            // Expected
        }
    }
}

==> ehcache_2.5.0.orig.tar.gz: ehcache-2.5.0.orig/src/test/java/net/sf/ehcache/constructs/blocking/BlockingCacheTest.java <==
/**
 *  Copyright 2003-2010 Terracotta, Inc.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package net.sf.ehcache.constructs.blocking;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheException;
import net.sf.ehcache.CacheTest;
import net.sf.ehcache.Ehcache;
import net.sf.ehcache.Element;
import net.sf.ehcache.Status;
import net.sf.ehcache.config.CacheConfiguration;
import net.sf.ehcache.statistics.LiveCacheStatistics;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

/**
 * Test cases for the {@link BlockingCache}.
 *
 * @author Adam Murdoch
 * @author Greg Luck
 * @version $Id: BlockingCacheTest.java 4769 2011-09-30 00:56:41Z asingh $
 */
public class BlockingCacheTest extends CacheTest {

    private BlockingCache blockingCache;

    /**
     * Load up the test cache
     */
    @Override
    @Before
    public void setUp() throws Exception {
        super.setUp();
        Ehcache cache = manager.getCache("sampleIdlingExpiringCache");
        if (cache != null) {
            blockingCache = new BlockingCache(cache);
        }
    }

    /**
     * teardown
     */
    @Override
    @After
    public void tearDown() throws Exception {
        if (manager.getStatus() == Status.STATUS_ALIVE) {
            if (blockingCache != null) {
                blockingCache.removeAll();
            }
        }
        super.tearDown();
    }

    @Test
    public void testSupportsStatsCorrectly() {
        blockingCache.setStatisticsEnabled(true);
        LiveCacheStatistics statistics = blockingCache.getLiveCacheStatistics();
        long cacheMisses = statistics.getCacheMissCount();
        long cacheHits = statistics.getCacheHitCount();
        String key = "123451234";
        blockingCache.get(key);
        assertEquals("Misses stat should have incremented by one", cacheMisses + 1, statistics.getCacheMissCount());
        assertEquals("Hits stat should have remain the same", cacheHits, statistics.getCacheHitCount());
        blockingCache.put(new Element(key, "value"));
        assertEquals("Misses stat should have incremented by one", cacheMisses + 1, statistics.getCacheMissCount());
        assertEquals("Hits stat should have remain the same", cacheHits, statistics.getCacheHitCount());
        assertNotNull(blockingCache.get(key));
        assertEquals("Hits stat should have incremented by one", cacheHits + 1, statistics.getCacheHitCount());
        blockingCache.setStatisticsEnabled(false);
        blockingCache.remove(key);
    }

    /**
     * Tests adding and looking up an entry.
     */
    @Test
    public void testAddEntry() throws Exception {
        //some other test was leaving this non-empty
        blockingCache.removeAll();

        final String key = "key";
        final String value = "value";
        Element element = new Element(key, value);

        // Check the cache is empty
        assertEquals(0, blockingCache.getKeys().size());

        // Put the entry
        blockingCache.put(new Element(key, value));

        // Check there is a single entry
        assertEquals(1, blockingCache.getKeys().size());
        assertTrue(blockingCache.getKeys().contains(key));
        final Element returnedElement = blockingCache.get(key);
        assertEquals(element, returnedElement);

    }

    /**
     * Tests that getting entries matches a list of known entries
     */
    @Test
    public void testGetEntries() throws Exception {
        Ehcache cache = blockingCache.getCache();
        for (int i = 0; i < 100; i++) {
            cache.put(new Element(Integer.valueOf(i), "value" + i));
        }
        List keys = blockingCache.getKeys();
        List elements = new ArrayList();
        for (int i = 0; i < keys.size(); i++) {
            Object key = keys.get(i);
            elements.add(blockingCache.get(key));
        }
        assertEquals(100, elements.size());
        Map map = new HashMap();
        for (int i = 0; i < elements.size(); i++) {
            Element element = (Element) elements.get(i);
            map.put(element.getObjectKey(), element.getObjectValue());
        }
        for (int i = 0; i < 100; i++) {
            Serializable value = (Serializable) map.get(Integer.valueOf(i));
            assertEquals("value" + i, value);
        }
    }

    /**
     * Tests looking up a missing entry, then adding it.
     */
    @Test
    public void testAddMissingEntry() throws Exception {
        Element element = new Element("key", "value");

        // Make sure the entry does not exist
        assertNull(blockingCache.get("key"));

        // Put the entry
        blockingCache.put(element);

        // Check the entry is in the cache
        assertEquals(1, blockingCache.getKeys().size());
        assertEquals(element, blockingCache.get("key"));

    }


    /**
     * Does a second tread block until the first thread puts the entry?
     */
    @Test
    public void testSecondThreadActuallyBlocks() throws Exception {
        Element element = new Element("key", "value");
        final List threadResults = new ArrayList();

        // Make sure the entry does not exist
        assertNull(blockingCache.get("key"));

        Thread secondThread = new Thread() {
            @Override
            public void run() {
                threadResults.add(blockingCache.get("key"));
            }
        };
        secondThread.start();
        assertEquals(0, threadResults.size());

        // Put the entry
        blockingCache.put(element);
        Thread.sleep(30);
        assertEquals(1, threadResults.size());
        assertEquals(element, threadResults.get(0));

        // Check the entry is in the cache
        assertEquals(1, blockingCache.getKeys().size());
        assertEquals(element, blockingCache.get("key"));
    }

    /**
     * Elements with null valuea are not stored in the blocking cache
     */
    @Test
    public void testUnknownEntry() throws Exception {
        // Make sure the entry does not exist
        assertNull(blockingCache.get("key"));
        // Put the entry
        blockingCache.put(new Element("key", null));
        assertEquals(0, blockingCache.getKeys().size());
    }

    /**
     * Overwriting an Element with an element with a null value effectively removes it from the cache
     */
    @Test
    public void testRemoveEntry() throws Exception {
        Element element = new Element("key", "value");

        // Add entry and make sure it's there
        blockingCache.put(element);
        assertEquals(element, blockingCache.get("key"));

        // Remove the entry and make sure its gone
        blockingCache.put(new Element("key", null));
        assertEquals(0, blockingCache.getKeys().size());

    }

    /**
     * Tests clearing the cache
     */
    @Test
    public void testClear() throws Exception {
        Ehcache cache = manager.getCache("sampleCacheNotEternalButNoIdleOrExpiry");
        blockingCache = new BlockingCache(cache);
        // Add some entries
        blockingCache.put(new Element("key1", "value1"));
        blockingCache.put(new Element("key2", "value2"));
        blockingCache.put(new Element("key3", "value2"));
        assertEquals(3, blockingCache.getKeys().size());

        // Clear the cache
        blockingCache.removeAll();
        assertEquals(0, blockingCache.getKeys().size());
    }


    /**
     * Creates a blocking test cache
     */
    @Override
    protected Ehcache createTestCache() {
        Ehcache cache = super.createTestCache();
        return new BlockingCache(cache);
    }

    /**
     * Gets the sample cache 1
     */
    @Override
    protected Ehcache getSampleCache1() {
        Cache cache = manager.getCache("sampleCache1");
        manager.replaceCacheWithDecoratedCache(cache, new BlockingCache(cache));
        return manager.getEhcache("sampleCache1");
    }

    /**
     * Use to manually test super class tests
     */
    @Test
    public void testInstrumented() throws Exception {
        super.testSizes();
    }

    @Override
    @Test
    public void testGetWithLoader() {
        super.testGetWithLoader();
    }


    @Override
    @Test
    public void testFlushWhenOverflowToDisk() throws Exception {
        super.testFlushWhenOverflowToDisk();
    }

    @Override
    @Test
    public void testConcurrentPutsAreConsistentRepeatedly() throws InterruptedException {
        //do nothing
    }

    @Override
    @Test
    public void testConcurrentPutsAreConsistent() throws InterruptedException {
        //do nothing
    }

    @Test
    public void testInlineEviction() throws InterruptedException {

        final Serializable KEY = "DUH";
        Cache cache = new Cache(new CacheConfiguration("fastExpiry", 1000).timeToIdleSeconds(2).timeToLiveSeconds(2));
        manager.addCache(cache);
        manager.replaceCacheWithDecoratedCache(cache, new BlockingCache(cache));

        Ehcache blockingCache = manager.getEhcache("fastExpiry");
        blockingCache.put(new Element(KEY, "VALUE"));
        assertNotNull(blockingCache.get(KEY));
        // This tests inline eviction (EHC-420)
        Thread.sleep(3000);
        assertNull(blockingCache.get(KEY));
    }

    @Test
    public void testTimeout() throws BrokenBarrierException, InterruptedException {
        final CyclicBarrier barrier = new CyclicBarrier(2);
        final String KEY = "BLOCKING_KEY";
        blockingCache.setTimeoutMillis(1000);
        Thread thread = new Thread(new Runnable() {
            public void run() {
                assertNull(blockingCache.get(KEY));
                try {
                    barrier.await();
                    Thread.sleep(5000);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
                blockingCache.put(new Element(KEY, "VALUE"));
            }
        });
        thread.start();
        barrier.await();
        try {
            blockingCache.get(KEY);
            fail("BlockingCache.get should have not returned!");
        } catch (CacheException e) {
            // Expected
        }
    }
}
